package com.nationrel.modules.rp.aop;

import com.nationrel.modules.rp.entity.PredictionInfo;
import com.nationrel.modules.rp.mapper.PredictionInfoMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.SecurityUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * @Description 更新项目信息AOP
 * 每次操作任何数据更新主表
 * @Author nijun
 * @Date 2024/9/2 15:21
 * @Version 1.0
 */
@Aspect
@Slf4j
@Component
public class UpdatePredictionAspect {
    @Autowired
    private PredictionInfoMapper predictionInfoMapper;

    @AfterReturning(value = "@annotation(updatePrediction)", returning = "predictionInfo")
    public void around(JoinPoint joinPoint, UpdatePrediction updatePrediction, PredictionInfo predictionInfo) throws Throwable {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        Date date = new Date();
        predictionInfo.setUpdateBy(sysUser.getUsername());
        predictionInfo.setUpdateTime(date);
        predictionInfo.setVersionNo(predictionInfo.getVersionNo()+1);
        predictionInfoMapper.updateById(predictionInfo);
        log.info("方法: {}, 更新可靠性预测主表信息, 用户： {}, 时间：{}", joinPoint.getSignature().getName(), sysUser.getUsername(), date);
    }
}
