package com.nationrel.modules.rp.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.nationrel.modules.rp.entity.PredictionTableCategoryRelation;
import com.nationrel.modules.rp.enums.CategpryEnums;
import com.nationrel.modules.rp.req.PredictionCategoryFormReq;
import com.nationrel.modules.rp.resp.PredictionCategoryFormResp;
import com.nationrel.modules.rp.resp.PredictionCategoryResp;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.jeecg.common.api.IPageExtField;
import org.jeecg.common.api.PageExtField;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import com.nationrel.modules.rp.entity.PredictionCategory;
import com.nationrel.modules.rp.service.IPredictionCategoryService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.config.i18n.MessageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.apache.shiro.authz.annotation.RequiresPermissions;

 /**
 * @Description: prediction_category
 * @Author: nationrel
 * @Date: 2024-09-06
 * @Version: V1.0
 */
@Tag(name="prediction_category")
@RestController
@RequestMapping("/rp/predictionCategory")
@Slf4j
public class PredictionCategoryController extends JeecgController<PredictionCategory, IPredictionCategoryService> {
	@Autowired
	private IPredictionCategoryService predictionCategoryService;

	/**
	 * 分页列表查询
	 *
	 * @param predictionCategory
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "prediction_category-分页列表查询")
	@Operation(summary="prediction_category-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPageExtField<PredictionCategory>> queryPageList(PredictionCategory predictionCategory,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<PredictionCategory> queryWrapper = QueryGenerator.initQueryWrapper(predictionCategory, req.getParameterMap());
		IPageExtField<PredictionCategory> page = new PageExtField<>(PredictionCategory.class, pageNo, pageSize);
		IPageExtField<PredictionCategory> pageList = predictionCategoryService.page(page, queryWrapper);
		return Result.OK(pageList);
	}

	/**
	 *   添加
	 *
	 * @param predictionCategory
	 * @return
	 */
	@AutoLog(value = "prediction_category-添加")
	@Operation(summary="prediction_category-添加")
	@RequiresPermissions("rp:prediction_category:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody PredictionCategory predictionCategory) {
		predictionCategoryService.save(predictionCategory);
		return Result.OK(MessageUtils.get("nationrel_add_success"));
	}

	/**
	 *  编辑
	 *
	 * @param predictionCategory
	 * @return
	 */
	@AutoLog(value = "prediction_category-编辑")
	@Operation(summary="prediction_category-编辑")
	@RequiresPermissions("rp:prediction_category:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody PredictionCategory predictionCategory) {
		predictionCategoryService.updateById(predictionCategory);
		return Result.OK(MessageUtils.get("nationrel_edit_success"));
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "prediction_category-通过id删除")
	@Operation(summary="prediction_category-通过id删除")
	@RequiresPermissions("rp:prediction_category:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		predictionCategoryService.removeById(id);
		return Result.OK(MessageUtils.get("nationrel_delete_success"));
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "prediction_category-批量删除")
	@Operation(summary="prediction_category-批量删除")
	@RequiresPermissions("rp:prediction_category:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.predictionCategoryService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK(MessageUtils.get("nationrel_delete_batch_success"));
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "prediction_category-通过id查询")
	@Operation(summary="prediction_category-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<PredictionCategory> queryById(@RequestParam(name="id",required=true) String id) {
		PredictionCategory predictionCategory = predictionCategoryService.getById(id);
		if(predictionCategory==null) {
			return Result.error(MessageUtils.get("nationrel_no_found_data"));
		}
		return Result.OK(predictionCategory);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param predictionCategory
    */
    @RequiresPermissions("rp:prediction_category:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, PredictionCategory predictionCategory) {
        return super.exportXls(request, predictionCategory, PredictionCategory.class, "prediction_category");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequiresPermissions("rp:prediction_category:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, PredictionCategory.class);
    }

	 @Operation(summary="prediction_category-查询类别下拉")
	 @GetMapping(value = "/getCategoryList")
	 public Result<PredictionCategoryResp> getCategoryList() {
		 PredictionCategoryResp predictionCategory = predictionCategoryService.getPredictionCategory();
		 return Result.OK(predictionCategory);
	 }

	 @Operation(summary="prediction_category-查询大类")
	 @GetMapping(value = "/getLargeCategory")
	 public Result<List<PredictionCategory>> getLargeCategory() {
		 List<PredictionCategory> predictionCategoryList = predictionCategoryService.list(new LambdaQueryWrapper<PredictionCategory>()
						 .eq(PredictionCategory::getCategoryType, CategpryEnums.LARGE.getValue())
				 .last("limit 10"));
		 return Result.OK(predictionCategoryList);
	 }

	 @Operation(summary="prediction_category-查询小类")
	 @GetMapping(value = "/getSmallCategory")
	 public Result<List<PredictionCategory>> getSmallCategory() {
		 List<PredictionCategory> predictionCategoryList = predictionCategoryService.list(new LambdaQueryWrapper<PredictionCategory>()
				 .eq(PredictionCategory::getCategoryType, CategpryEnums.SMALL.getValue())
				 .last("limit 10"));
		 return Result.OK(predictionCategoryList);
	 }

	 @Operation(summary="prediction_category-获取表单")
	 @PostMapping(value = "/getCategoryForm")
	 public Result<PredictionCategoryFormResp> getCategoryForm(@RequestBody PredictionCategoryFormReq predictionCategoryFormReq) {
		 PredictionCategoryFormResp categoryForm = predictionCategoryService.getCategoryForm(predictionCategoryFormReq);
		 return Result.OK(categoryForm);
	 }

	 @Operation(summary="prediction_category-保存表单")
	 @PostMapping(value = "/saveOrUpdateRelation")
	 public Result<String> saveOrUpdateRelation(@RequestBody PredictionTableCategoryRelation predictionTableCategoryRelation) {
		predictionCategoryService.saveOrUpdateRelation(predictionTableCategoryRelation);
		return Result.OK(MessageUtils.get("nationrel_operation_success"));
	 }
}
