package com.nationrel.modules.rp.controller;

import java.util.Arrays;

import com.nationrel.modules.rp.entity.PredictionInfo;
import com.nationrel.modules.rp.service.IPredictionInfoService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.jeecg.common.api.IPageExtField;
import org.jeecg.common.api.PageExtField;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.config.i18n.MessageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.apache.shiro.authz.annotation.RequiresPermissions;

 /**
 * @Description: prediction_info
 * @Author: nationrel
 * @Date: 2024-08-26
 * @Version: V1.0
 */
@Tag(name="prediction_info")
@RestController
@RequestMapping("/rp/predictionInfo")
@Slf4j
public class PredictionInfoController extends JeecgController<PredictionInfo, IPredictionInfoService> {
	@Autowired
	private IPredictionInfoService predictionInfoService;

	/**
	 * 分页列表查询
	 *
	 * @param predictionInfo
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	//@AutoLog(value = "prediction_info-分页列表查询")
	@Operation(summary="prediction_info-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPageExtField<PredictionInfo>> queryPageList(PredictionInfo predictionInfo,
								   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
								   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
								   HttpServletRequest req) {
		QueryWrapper<PredictionInfo> queryWrapper = QueryGenerator.initQueryWrapper(predictionInfo, req.getParameterMap());
		IPageExtField<PredictionInfo> page = new PageExtField<>(PredictionInfo.class, pageNo, pageSize);
		IPageExtField<PredictionInfo> pageList = predictionInfoService.page(page, queryWrapper);
		return Result.OK(pageList);
	}

	/**
	 *   添加
	 *
	 * @param predictionInfo
	 * @return
	 */
	@AutoLog(value = "prediction_info-添加")
	@Operation(summary="prediction_info-添加")
	@RequiresPermissions("rp:prediction_info:add")
	@PostMapping(value = "/add")
	public Result<String> add(@RequestBody PredictionInfo predictionInfo) {
		predictionInfoService.savePredictionInfo(predictionInfo);
		return Result.OK(MessageUtils.get("nationrel_add_success"));
	}

	/**
	 *  编辑
	 *
	 * @param predictionInfo
	 * @return
	 */
	@AutoLog(value = "prediction_info-编辑")
	@Operation(summary="prediction_info-编辑")
	@RequiresPermissions("rp:prediction_info:edit")
	@RequestMapping(value = "/edit", method = {RequestMethod.PUT,RequestMethod.POST})
	public Result<String> edit(@RequestBody PredictionInfo predictionInfo) {
		predictionInfoService.editPredictionInfo(predictionInfo);
		return Result.OK(MessageUtils.get("nationrel_edit_success"));
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "prediction_info-通过id删除")
	@Operation(summary="prediction_info-通过id删除")
	@RequiresPermissions("rp:prediction_info:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		predictionInfoService.removeById(id);
		return Result.OK(MessageUtils.get("nationrel_delete_success"));
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "prediction_info-批量删除")
	@Operation(summary="prediction_info-批量删除")
	@RequiresPermissions("rp:prediction_info:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.predictionInfoService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK(MessageUtils.get("nationrel_delete_batch_success"));
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "prediction_info-通过id查询")
	@Operation(summary="prediction_info-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<PredictionInfo> queryById(@RequestParam(name="id",required=true) String id) {
		PredictionInfo predictionInfo = predictionInfoService.getById(id);
		if(predictionInfo==null) {
			return Result.error(MessageUtils.get("nationrel_no_found_data"));
		}
		return Result.OK(predictionInfo);
	}

    /**
    * 导出excel
    *
    * @param request
    * @param predictionInfo
    */
    @RequiresPermissions("rp:prediction_info:exportXls")
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, PredictionInfo predictionInfo) {
        return super.exportXls(request, predictionInfo, PredictionInfo.class, "prediction_info");
    }

    /**
      * 通过excel导入数据
    *
    * @param request
    * @param response
    * @return
    */
    @RequiresPermissions("rp:prediction_info:importExcel")
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        return super.importExcel(request, response, PredictionInfo.class);
    }

}
