package com.nationrel.modules.rp.controller;

import java.util.Arrays;
import java.util.List;

import com.nationrel.modules.rp.entity.PredictionStruTree;
import com.nationrel.modules.rp.req.DelStructureReq;
import com.nationrel.modules.rp.req.PasteStructureReq;
import com.nationrel.modules.rp.req.SaveStructureReq;
import com.nationrel.modules.rp.req.UpdateNodeNameReq;
import com.nationrel.modules.rp.resp.PredictionStruTreeResp;
import com.nationrel.modules.rp.resp.QueryPredictionStruTreeResp;
import com.nationrel.modules.rp.service.IPredictionStruTreeService;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.jeecg.common.api.IPageExtField;
import org.jeecg.common.api.PageExtField;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.config.i18n.MessageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.ModelAndView;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.apache.shiro.authz.annotation.RequiresPermissions;

 /**
 * @Description: prediction_stru_tree
 * @Author: nationrel
 * @Date: 2024-08-26
 * @Version: V1.0
 */
@Tag(name="prediction_stru_tree")
@RestController
@RequestMapping("/rp/predictionStruTree")
@Slf4j
public class PredictionStruTreeController extends JeecgController<PredictionStruTree, IPredictionStruTreeService> {
	@Autowired
	private IPredictionStruTreeService predictionStruTreeService;

	/**
	 * @Description 获取左侧树
	 * @param predictionId
	 * @return org.jeecg.common.api.vo.Result<java.util.List<com.nationrel.modules.rp.resp.PredictionStruTreeResp>>
	 * @Author nijun
	 * @Date: 2024/8/27 17:59
	 */
	@RequestMapping(value = "/getLeftTree", method = RequestMethod.GET)
	public Result<QueryPredictionStruTreeResp> getLeftTree(Long predictionId) {
		QueryPredictionStruTreeResp struTreeResp = predictionStruTreeService.getLeftTree(predictionId, null);
		return Result.OK(struTreeResp);
	}

	 @AutoLog(value = "prediction_stru_tree-添加")
	 @Operation(summary="prediction_stru_tree-添加")
	 @PostMapping(value = "/saveStructure")
	 public Result<String> saveStructure(@RequestBody SaveStructureReq saveStructureReq) {
		 predictionStruTreeService.addStructure(saveStructureReq);
		 return Result.OK(MessageUtils.get("nationrel_add_success"));
	 }

	 @AutoLog(value = "prediction_stru_tree-删除")
	 @Operation(summary="prediction_stru_tree-删除")
	 @DeleteMapping(value = "/delStructure")
	 public Result<String> saveStructure(@RequestParam(name="predictionId",required=true) String predictionId,
										 @RequestParam(name="struId",required=true) String struId) {
		 predictionStruTreeService.deleteStructure(predictionId, struId);
		 return Result.OK(MessageUtils.get("nationrel_add_success"));
	 }

	 @AutoLog(value = "prediction_stru_tree-粘贴")
	 @Operation(summary="prediction_stru_tree-粘贴")
	 @PostMapping(value = "/pasteStructure")
	 public Result<String> paste(@RequestBody PasteStructureReq pasteStructureReq) throws Exception {
		 predictionStruTreeService.pasteStructure(pasteStructureReq);
		 return Result.OK(MessageUtils.get("nationrel_add_success"));
	 }

	 @AutoLog(value = "prediction_stru_tree-修改节点名称")
	 @Operation(summary="prediction_stru_tree-修改节点名称")
	 @PostMapping(value = "/updateNodeName")
	 public Result<String> updateNodeName(@RequestBody UpdateNodeNameReq updateNodeNameReq) throws Exception {
		 predictionStruTreeService.updateNodeName(updateNodeNameReq);
		 return Result.OK(MessageUtils.get("nationrel_add_success"));
	 }
}
