package com.nationrel.modules.rp.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.apache.commons.lang3.ObjectUtils;
import org.jeecg.common.api.IPageExtField;
import org.jeecg.common.api.PageExtField;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import com.nationrel.modules.rp.entity.PredictionTable;
import com.nationrel.modules.rp.service.IPredictionTableService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.jeecg.common.system.base.controller.JeecgController;
import org.jeecg.config.i18n.MessageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.apache.shiro.authz.annotation.RequiresPermissions;

 /**
 * @Description: prediction_table
 * @Author: nationrel
 * @Date: 2024-09-02
 * @Version: V1.0
 */
@Tag(name="prediction_table")
@RestController
@RequestMapping("/rp/predictionTable")
@Slf4j
public class PredictionTableController extends JeecgController<PredictionTable, IPredictionTableService> {
	@Autowired
	private IPredictionTableService predictionTableService;

	/**
	 *   添加
	 *
	 * @param predictionTable
	 * @return
	 */
	@AutoLog(value = "prediction_table-添加/更新")
	@Operation(summary="prediction_table-添加/更新")
	@PostMapping(value = "/saveOrUpdate")
	public Result<String> add(@RequestBody PredictionTable predictionTable) {
		predictionTableService.saveOrUpdateTable(predictionTable);
		return Result.OK(MessageUtils.get("nationrel_add_success"));
	}

	/**
	 *   通过id删除
	 *
	 * @param id
	 * @return
	 */
	@AutoLog(value = "prediction_table-通过id删除")
	@Operation(summary="prediction_table-通过id删除")
	@RequiresPermissions("rp:prediction_table:delete")
	@DeleteMapping(value = "/delete")
	public Result<String> delete(@RequestParam(name="id",required=true) String id) {
		predictionTableService.removeById(id);
		return Result.OK(MessageUtils.get("nationrel_delete_success"));
	}

	/**
	 *  批量删除
	 *
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "prediction_table-批量删除")
	@Operation(summary="prediction_table-批量删除")
	@RequiresPermissions("rp:prediction_table:deleteBatch")
	@DeleteMapping(value = "/deleteBatch")
	public Result<String> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		this.predictionTableService.removeByIds(Arrays.asList(ids.split(",")));
		return Result.OK(MessageUtils.get("nationrel_delete_batch_success"));
	}

	/**
	 * 通过id查询
	 *
	 * @param id
	 * @return
	 */
	//@AutoLog(value = "prediction_table-通过id查询")
	@Operation(summary="prediction_table-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<PredictionTable> queryById(@RequestParam(name="id",required=true) String id) {
		PredictionTable predictionTable = predictionTableService.getById(id);
		if(predictionTable==null) {
			return Result.error(MessageUtils.get("nationrel_no_found_data"));
		}
		return Result.OK(predictionTable);
	}

	 @Operation(summary="prediction_table-获取表格行ID")
	 @GetMapping(value = "/getTableId")
	public Result<String> getTableId(@RequestParam(name="predictionId",required=true) String predictionId, @RequestParam(name="struId",required=true) String struId) {
		 PredictionTable predictionTable = predictionTableService.getOne(new LambdaQueryWrapper<PredictionTable>()
				 .eq(PredictionTable::getPredictionId, predictionId)
				 .eq(PredictionTable::getStruId, struId));
		 if (ObjectUtils.isNotEmpty(predictionTable)) {
			 return Result.ok(predictionTable.getId());
		 }
		 return Result.error(MessageUtils.get("nationrel_no_found_data"));
	 }

}
