package com.nationrel.modules.rp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.j2objc.annotations.AutoreleasePool;
import com.nationrel.modules.rp.aop.UpdatePrediction;
import com.nationrel.modules.rp.entity.PredictionCategory;
import com.nationrel.modules.rp.entity.PredictionCategoryOptions;
import com.nationrel.modules.rp.entity.PredictionInfo;
import com.nationrel.modules.rp.entity.PredictionTableCategoryRelation;
import com.nationrel.modules.rp.enums.CategpryEnums;
import com.nationrel.modules.rp.mapper.PredictionCategoryMapper;
import com.nationrel.modules.rp.mapper.PredictionCategoryOptionsMapper;
import com.nationrel.modules.rp.mapper.PredictionInfoMapper;
import com.nationrel.modules.rp.mapper.PredictionTableCategoryRelationMapper;
import com.nationrel.modules.rp.model.PredictionCategoryChildrenModel;
import com.nationrel.modules.rp.model.PredictionCategoryModel;
import com.nationrel.modules.rp.req.PredictionCategoryFormReq;
import com.nationrel.modules.rp.resp.PredictionCategoryFormResp;
import com.nationrel.modules.rp.resp.PredictionCategoryResp;
import com.nationrel.modules.rp.service.IPredictionCategoryService;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: prediction_category
 * @Author: nationrel
 * @Date:   2024-09-06
 * @Version: V1.0
 */
@Service
public class PredictionCategoryServiceImpl extends ServiceImpl<PredictionCategoryMapper, PredictionCategory> implements IPredictionCategoryService {

    @Autowired
    private PredictionInfoMapper predictionInfoMapper;;
    @Autowired
    private PredictionCategoryMapper predictionCategoryMapper;
    @Autowired
    private PredictionCategoryOptionsMapper predictionCategoryOptionsMapper;
    @Autowired
    private PredictionTableCategoryRelationMapper predictionTableCategoryRelationMapper;

    /**
     * @Description 获取类别
     * @param
     * @return com.nationrel.modules.rp.resp.PredictionCategoryResp
     * @Author nijun
     * @Date: 2024/9/6 16:50
     */
    @Override
    public PredictionCategoryResp getPredictionCategory() {
        List<PredictionCategory> largeCategory = predictionCategoryMapper.selectList(new LambdaQueryWrapper<PredictionCategory>()
                .eq(PredictionCategory::getCategoryType, CategpryEnums.LARGE.getValue()));

        List<PredictionCategory> smallCategory = predictionCategoryMapper.selectList(new LambdaQueryWrapper<PredictionCategory>()
                .eq(PredictionCategory::getCategoryType, CategpryEnums.SMALL.getValue()));

        PredictionCategoryResp predictionCategoryResp = new PredictionCategoryResp();
        predictionCategoryResp.setLargeCategoryList(largeCategory);
        predictionCategoryResp.setSmallCategoryList(smallCategory);

        return predictionCategoryResp;
    }

    /**
     * @Description 获取类别下的表单
     * @param predictionCategoryFormReq
     * @return com.nationrel.modules.rp.resp.PredictionCategoryFormResp
     * @Author nijun
     * @Date: 2024/9/8 23:23
     */
    @Override
    public PredictionCategoryFormResp getCategoryForm(PredictionCategoryFormReq predictionCategoryFormReq) {
        PredictionCategoryFormResp predictionCategoryFormResp = new PredictionCategoryFormResp();
        // 关系值
        List<PredictionTableCategoryRelation> relationList = predictionTableCategoryRelationMapper.selectList(new LambdaQueryWrapper<PredictionTableCategoryRelation>()
                .eq(PredictionTableCategoryRelation::getPredictionId, predictionCategoryFormReq.getPredictionId())
                .eq(PredictionTableCategoryRelation::getStruId, predictionCategoryFormReq.getStruId()));
        Map<String, String> relationMap = relationList.stream().collect(Collectors.toMap(PredictionTableCategoryRelation::getCategoryId, PredictionTableCategoryRelation::getCategoryValue));
        // 小类
//        PredictionCategory smallCategory = predictionCategoryMapper.selectOne(new LambdaQueryWrapper<PredictionCategory>()
//                .eq(PredictionCategory::getCode, predictionCategoryFormReq.getCode()));
        // 表单标签
        List<PredictionCategory> labelCategory = predictionCategoryMapper.selectList(new LambdaQueryWrapper<PredictionCategory>()
                .eq(PredictionCategory::getParentCode, predictionCategoryFormReq.getCode()));
        // 输出
        predictionCategoryFormResp.setOutputForm(labelCategory.stream().filter(item ->
                item.getIoType().equals(CategpryEnums.OUTPUT.getValue())).toList());

        List<PredictionCategoryModel> formList = labelCategory.stream().filter(item ->
                item.getIoType().equals(CategpryEnums.INPUT.getValue())).map(item -> {
            PredictionCategoryModel predictionCategoryModel = new PredictionCategoryModel();
            BeanUtils.copyProperties(item, predictionCategoryModel);

            if (item.getCategoryType().equals(CategpryEnums.CHILDREN_FORM.getValue())) {
                List<PredictionCategory> childList = predictionCategoryMapper.selectList(new LambdaQueryWrapper<PredictionCategory>()
                        .eq(PredictionCategory::getParentCode, item.getCode())
                        .eq(PredictionCategory::getIoType, CategpryEnums.INPUT.getValue()));
                List<PredictionCategoryModel> childrenModelList = childList.stream().map(select -> {
                    PredictionCategoryModel childPredictionCategoryModel = new PredictionCategoryModel();
                    BeanUtils.copyProperties(select, childPredictionCategoryModel);
                    this.getChildForm(select.getCode(), childPredictionCategoryModel);
//                    childPredictionCategoryModel.setChildren(Lists.newArrayList());
                    return childPredictionCategoryModel;
                }).toList();
                predictionCategoryModel.setChildrenForm(childrenModelList);
                predictionCategoryModel.setChildren(Lists.newArrayList());
            } else {
                this.getChildForm(item.getCode(), predictionCategoryModel);
            }

            if (StringUtils.isNotEmpty(relationMap.get(item.getId()))) {
                predictionCategoryModel.setCategoryValue(relationMap.get(item.getId()));
            }
            return predictionCategoryModel;
        }).toList();
        predictionCategoryFormResp.setInputForm(formList);
        return predictionCategoryFormResp;
    }

    private PredictionCategoryModel getChildForm(String code, PredictionCategoryModel predictionCategoryModel) {
        List<PredictionCategory> selectList = predictionCategoryMapper.selectList(new LambdaQueryWrapper<PredictionCategory>()
                .eq(PredictionCategory::getParentCode, code)
                .eq(PredictionCategory::getIoType, CategpryEnums.INPUT.getValue()));
        selectList.forEach(child -> {
            if (child.getCategoryType().equals(CategpryEnums.SELECT.getValue()) ||
                    child.getCategoryType().equals(CategpryEnums.FIELD.getValue())) {
                List<PredictionCategoryChildrenModel> childrenModelList = selectList.stream().map(select -> {
                    PredictionCategoryChildrenModel categoryChildrenModel = new PredictionCategoryChildrenModel();
                    categoryChildrenModel.setCode(select.getCode());
                    categoryChildrenModel.setName(select.getValue());
                    categoryChildrenModel.setSelectPlaceholder(select.getSelectPlaceholder());
                    categoryChildrenModel.setParentCode(select.getParentCode());
                    categoryChildrenModel.setRelationItem(select.getRelationItem());
                    return categoryChildrenModel;
                }).toList();
                predictionCategoryModel.setChildren(childrenModelList);
            } else if (child.getCategoryType().equals(CategpryEnums.OPTIONS.getValue())) {
                List<PredictionCategoryOptions> predictionCategoryOptions = predictionCategoryOptionsMapper.selectList(new LambdaQueryWrapper<PredictionCategoryOptions>()
                        .eq(PredictionCategoryOptions::getParentCode, child.getValue()));
                List<PredictionCategoryChildrenModel> childrenModelList = predictionCategoryOptions.stream().map(options -> {
                    PredictionCategoryChildrenModel categoryChildrenModel = new PredictionCategoryChildrenModel();
                    categoryChildrenModel.setCode(options.getCode());
                    categoryChildrenModel.setName(options.getName());
                    categoryChildrenModel.setReference(options.getReference());
                    categoryChildrenModel.setParentCode(options.getParentCode());
                    categoryChildrenModel.setSelectPlaceholder(options.getPlaceholder());
                    return categoryChildrenModel;
                }).toList();
                predictionCategoryModel.setChildren(childrenModelList);
            }
        });
        return predictionCategoryModel;
    }

    /**
     * @Description 保存/更新类别表单关系表
     * @param predictionTableCategoryRelation
     * @return PredictionInfo
     * @Author nijun
     * @Date: 2024/9/10 16:41
     */
    @Override
    @UpdatePrediction
    @Transactional(rollbackFor = Exception.class)
    public PredictionInfo saveOrUpdateRelation(PredictionTableCategoryRelation predictionTableCategoryRelation) {
        PredictionInfo predictionInfo = predictionInfoMapper.selectById(predictionTableCategoryRelation.getPredictionId());
        PredictionTableCategoryRelation relation = predictionTableCategoryRelationMapper.selectOne(new LambdaQueryWrapper<PredictionTableCategoryRelation>()
                .eq(PredictionTableCategoryRelation::getPredictionId, predictionTableCategoryRelation.getPredictionId())
                .eq(PredictionTableCategoryRelation::getCategoryId, predictionTableCategoryRelation.getCategoryId())
                .eq(PredictionTableCategoryRelation::getStruId, predictionTableCategoryRelation.getStruId()));
        if (ObjectUtils.isNotEmpty(relation)) {
            relation.setCategoryValue(predictionTableCategoryRelation.getCategoryValue());
            predictionTableCategoryRelationMapper.updateById(relation);
        } else {
            predictionTableCategoryRelationMapper.insert(predictionTableCategoryRelation);
        }
        return predictionInfo;
    }
}
