package com.nationrel.modules.rp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.nationrel.modules.rp.constant.RpConstant;
import com.nationrel.modules.rp.entity.PredictionColumns;
import com.nationrel.modules.rp.entity.PredictionInfo;
import com.nationrel.modules.rp.entity.PredictionStruTree;
import com.nationrel.modules.rp.entity.PredictionTable;
import com.nationrel.modules.rp.enums.StateEnum;
import com.nationrel.modules.rp.mapper.PredictionInfoMapper;
import com.nationrel.modules.rp.mapper.PredictionStruTreeMapper;
import com.nationrel.modules.rp.mapper.PredictionTableMapper;
import com.nationrel.modules.rp.model.ColumnsModel;
import com.nationrel.modules.rp.service.IPredictionColumnsService;
import com.nationrel.modules.rp.service.IPredictionInfoService;
import me.zhyd.oauth.utils.UuidUtils;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang.math.RandomUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.exception.NationrelBootException;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @Description: prediction_info
 * @Author: nationrel
 * @Date:   2024-08-26
 * @Version: V1.0
 */
@Service
public class PredictionInfoServiceImpl extends ServiceImpl<PredictionInfoMapper, PredictionInfo> implements IPredictionInfoService {

    @Autowired
    private PredictionInfoMapper predictionInfoMapper;
    @Autowired
    private PredictionStruTreeMapper predictionStruTreeMapper;
    @Autowired
    private IPredictionColumnsService predictionColumnsService;
    @Autowired
    private PredictionTableMapper predictionTableMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void savePredictionInfo(PredictionInfo predictionInfo) {

        Long count = predictionInfoMapper.selectCount(new LambdaQueryWrapper<PredictionInfo>()
                .eq(PredictionInfo::getRpCode, predictionInfo.getRpCode())
                .eq(PredictionInfo::getDelFlag, StateEnum.NOT_DELETE.getValue()));
        if (count > 0) {
            throw new NationrelBootException("编号已存在");
        }

        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        predictionInfo.setCreateBy(sysUser.getUsername());
        predictionInfo.setCreateTime(new Date());
        predictionInfo.setRpCode(predictionInfo.getRpCode());
        predictionInfo.setDelFlag(StateEnum.NOT_DELETE.getValue());
        predictionInfo.setVersionNo(1);
        predictionInfoMapper.insert(predictionInfo);
        String struId = UuidUtils.getUUID();
        PredictionStruTree predictionStruTree = new PredictionStruTree();
        predictionStruTree.setStruId(struId);
        predictionStruTree.setParentStruId("00000000000000000000000000000000");
        predictionStruTree.setPredictionId(predictionInfo.getId());
        predictionStruTree.setStruTypeCode("01");
        predictionStruTree.setSeriesNo("1");
        predictionStruTree.setDeep(1);
        predictionStruTree.setLeftNo(1);
        predictionStruTree.setRightNo(2);
        predictionStruTree.setType("1");
        predictionStruTree.setName(predictionInfo.getRpName());
        predictionStruTree.setInsertTime(new Date());
        predictionStruTree.setInsertUserBy(sysUser.getUsername());
        predictionStruTreeMapper.insert(predictionStruTree);
        this.saveColumns(predictionInfo.getId(), sysUser.getUsername());
        PredictionColumns columns = predictionColumnsService.getOne(new LambdaQueryWrapper<PredictionColumns>()
                .eq(PredictionColumns::getPredictionId, predictionInfo.getId())
                .eq(PredictionColumns::getColSort, 1));
        PredictionTable predictionTable = new PredictionTable();
        predictionTable.setPredictionId(predictionInfo.getId());
        predictionTable.setStruId(struId);
        predictionTable.setCreateBy(sysUser.getUsername());
        predictionTable.setCreateTime(new Date());
        predictionTable.setColumnsId(columns.getId());
        predictionTable.setValue(RpConstant.DEFAULT_COL_VALUE);
        predictionTableMapper.insert(predictionTable);
    }

    @Override
    public void editPredictionInfo(PredictionInfo predictionInfo) {
        Long count = predictionInfoMapper.selectCount(new LambdaQueryWrapper<PredictionInfo>()
                .eq(PredictionInfo::getRpCode, predictionInfo.getRpCode())
                .ne(PredictionInfo::getId, predictionInfo.getId())
                .eq(PredictionInfo::getDelFlag, StateEnum.NOT_DELETE.getValue()));
        if (count > 0) {
            throw new NationrelBootException("编号已存在");
        }
        predictionInfoMapper.updateById(predictionInfo);
    }

    @Override
    public void updatePredictionInfo(PredictionInfo predictionInfo) {

    }

    @Override
    public void deletePredictionInfo(PredictionInfo predictionInfo) {

    }

    /**
     * @Description 保存默认的表格列头
     * @param predictionId
     * @param userName
     * @return void
     * @Author nijun
     * @Date: 2024/9/2 16:45
     */
    private void saveColumns(String predictionId, String userName) {
        List<PredictionColumns> columnsList = Lists.newArrayList();
        int index = 1;
        for (ColumnsModel column : RpConstant.DEFAULT_COLUMNS) {
            PredictionColumns predictionColumns = new PredictionColumns();
            predictionColumns.setPredictionId(predictionId);
            predictionColumns.setColName(column.getColumnName());
            predictionColumns.setColType(column.getColumnType());
            predictionColumns.setColSort(index++);
            predictionColumns.setCreateBy(userName);
            predictionColumns.setCreateTime(new Date());
            predictionColumns.setDisplayFlag("0");
            columnsList.add(predictionColumns);
        }
        predictionColumnsService.saveBatch(columnsList);
    }


    /**
     * @Description 编码
     * @param
     * @return java.lang.String
     * @Author nijun
     * @Date: 2024/8/26 19:13
     */
    public String executeCode() {
        String prefix = "RP";
        SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");
        int random = RandomUtils.nextInt(90) + 10;
        String value = prefix + format.format(new Date()) + random;
        return value;
    }
}
