package com.nationrel.modules.rp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.nationrel.modules.rp.aop.UpdatePrediction;
import com.nationrel.modules.rp.constant.RpConstant;
import com.nationrel.modules.rp.entity.PredictionColumns;
import com.nationrel.modules.rp.entity.PredictionInfo;
import com.nationrel.modules.rp.entity.PredictionTable;
import com.nationrel.modules.rp.mapper.PredictionColumnsMapper;
import com.nationrel.modules.rp.mapper.PredictionInfoMapper;
import com.nationrel.modules.rp.mapper.PredictionTableMapper;
import com.nationrel.modules.rp.service.IPredictionTableService;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.system.vo.LoginUser;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: prediction_table
 * @Author: nationrel
 * @Date:   2024-09-02
 * @Version: V1.0
 */
@Service
public class PredictionTableServiceImpl extends ServiceImpl<PredictionTableMapper, PredictionTable> implements IPredictionTableService {

    @Autowired
    private PredictionTableMapper predictionTableMapper;
    @Autowired
    private PredictionInfoMapper predictionInfoMapper;
    @Autowired
    private PredictionColumnsMapper predictionColumnsMapper;

    /**
     * @Description 保存表格数据
     * @param predictionTable
     * @return PredictionInfo
     * @Author nijun
     * @Date: 2024/9/8 23:23
     */
    @Override
    @UpdatePrediction
    @Transactional(rollbackFor = Exception.class)
    public PredictionInfo saveOrUpdateTable(PredictionTable predictionTable) {
        LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        PredictionInfo predictionInfo = predictionInfoMapper.selectById(predictionTable.getPredictionId());
        List<PredictionColumns> columnsList = predictionColumnsMapper.selectList(new LambdaQueryWrapper<PredictionColumns>()
                .eq(PredictionColumns::getPredictionId, predictionTable.getPredictionId()));
        Map<String, String> colMap = columnsList.stream().collect(Collectors.toMap(PredictionColumns::getColName, PredictionColumns::getId));
        PredictionTable table = predictionTableMapper.selectOne(new LambdaQueryWrapper<PredictionTable>()
                .eq(PredictionTable::getPredictionId, predictionTable.getPredictionId())
                .eq(PredictionTable::getStruId, predictionTable.getStruId())
                .eq(PredictionTable::getColumnsId, predictionTable.getColumnsId()));
        if (ObjectUtils.isNotEmpty(table)) {
            table.setValue(predictionTable.getValue());
            table.setUpdateBy(sysUser.getUsername());
            table.setUpdateTime(new Date());
            table.setVersionNo(table.getVersionNo()+1);
            predictionTableMapper.updateById(table);
            // 修改产品层次就清空后面所有列的数据
            List<PredictionTable> predictionTables = predictionTableMapper.selectList(new LambdaQueryWrapper<PredictionTable>()
                    .eq(PredictionTable::getPredictionId, predictionTable.getPredictionId())
                    .eq(PredictionTable::getStruId, predictionTable.getStruId()));

            if (predictionTable.getColumnsId().equals(colMap.get(RpConstant.PRODUCT_HIERARCHY)) &&
                    !predictionTable.getValue().equals(RpConstant.PRODUCT_HIERARCHY_COMPONENT_VALUE)) {
                List<String> idList = predictionTables.stream().filter(tables ->
                                !tables.getColumnsId().equals(colMap.get(RpConstant.PRODUCT_HIERARCHY)))
                        .map(PredictionTable::getId).toList();
                predictionTableMapper.deleteBatchIds(idList);
            } else {
                predictionTables.forEach(tb -> {
                    // 清除大类小类
                    if ((predictionTable.getColumnsId().equals(colMap.get(RpConstant.FUNCTION)) &&
                            (tb.getColumnsId().equals(colMap.get(RpConstant.LARGE)) || tb.getColumnsId().equals(colMap.get(RpConstant.SMALL))))) {
                        predictionTableMapper.deleteById(tb.getId());
                    }
                    // 清除小类
                    if ((predictionTable.getColumnsId().equals(colMap.get(RpConstant.LARGE))) && tb.getColumnsId().equals(colMap.get(RpConstant.SMALL))) {
                        predictionTableMapper.deleteById(tb.getId());
                    }
                    // 清除作用关系
                    if (predictionTable.getColumnsId().equals(colMap.get(RpConstant.COUNT)) && Integer.parseInt(predictionTable.getValue()) <= 1 &&
                            tb.getColumnsId().equals(colMap.get(RpConstant.RELATIONSHIP))) {
                        predictionTableMapper.deleteById(tb.getId());
                    } else if (predictionTable.getColumnsId().equals(colMap.get(RpConstant.COUNT)) && Integer.parseInt(predictionTable.getValue()) > 1) {
                        Long count = predictionTableMapper.selectCount(new LambdaQueryWrapper<PredictionTable>()
                                .eq(PredictionTable::getPredictionId, predictionTable.getPredictionId())
                                .eq(PredictionTable::getStruId, predictionTable.getStruId())
                                .eq(PredictionTable::getColumnsId, colMap.get(RpConstant.RELATIONSHIP)));
                        if (count == 0) {
                            PredictionTable countTable = new PredictionTable();
                            BeanUtils.copyProperties(predictionTable, countTable);
                            countTable.setId(null);
                            countTable.setColumnsId(colMap.get(RpConstant.RELATIONSHIP));
                            countTable.setValue(RpConstant.DEFAULT_RELATIONSHIP_VALUE);
                            predictionTableMapper.insert(countTable);
                        }
                    }
                });
            }
        } else {
            predictionTable.setCreateBy(sysUser.getUsername());
            predictionTable.setCreateTime(new Date());
            predictionTableMapper.insert(predictionTable);
            if (colMap.get(RpConstant.PRODUCT_HIERARCHY).equals(predictionTable.getColumnsId())) {
                PredictionColumns predictionColumns = predictionColumnsMapper.selectOne(new LambdaQueryWrapper<PredictionColumns>()
                        .eq(PredictionColumns::getPredictionId, predictionTable.getPredictionId())
                        .eq(PredictionColumns::getColName, RpConstant.DEFAULT_COLUMNS.get(5).getColumnName()));
                PredictionTable countTable = new PredictionTable();
                BeanUtils.copyProperties(predictionTable, countTable);
                countTable.setId(null);
                countTable.setColumnsId(predictionColumns.getId());
                countTable.setValue(RpConstant.DEFAULT_COUNT);
                predictionTableMapper.insert(countTable);
            }
        }
        return predictionInfo;
    }
}
